-- Import other files variables
local _, L = ...;

-- Addon config
MACROCOLLECTION = {
  -- Addon Title
  TITLE = "Macro Collection";
  ICON = "Interface\\MacroFrame\\MacroFrame-Icon";

  -- Tab Names
  CLASS = L["My Class"];
  NOCLASS = L["Generics"];
  ALL = L["All"];

  -- Default 'selected' values
  SELECTEDTAB = 1;
  SELECTEDNAME = L["Selected Macro"];
  SELECTEDDESC = L["Selected Macro's description will be displayed here."];
  SELECTEDMACRO = {};

  -- Others
  ACCOUNT = L["Account"];
  MACROS = {};
  ALLMACROS = {};
  BUTTONTOSCROLL = "";
  IDSPACER = "00100";

  -- Settings
  SETTINGS = {
    debug = false;
    MinimapPos = 45;
  };
};

-- SETTINGS
MACROCOLLECTION.LocalizedClassName, MACROCOLLECTION.PlayerClassName, MACROCOLLECTION.ClassIndex = UnitClass("Player");

-- Some settings
UIPanelWindows["MacroCollection"] = { area = "left", pushable = 1, whileDead = 1 };


-- ACTIONS
function MacroCollection_Toggle()
  if (MacroCollection:IsShown() ) then
    MacroCollection_Hide();
  else
    MacroCollection_Show();
  end
end
function MacroCollection_Hide()
  MacroCollection:Hide();
end
function MacroCollection_Show()
  MacroCollection:Show();
end

function MacroCollection_UpdatePageContent(selectedTab)
  -- Clean page content
  MacroCollection_ResetContainer();

  -- Update selected list
  MACROCOLLECTION.SELECTEDTAB = selectedTab;
  PanelTemplates_SetTab(MacroCollection, MACROCOLLECTION.SELECTEDTAB);

  if (selectedTab == 1) then
    MACROCOLLECTION.MACROS = _G["MacroCollection_" .. MACROCOLLECTION.PlayerClassName];

  elseif (selectedTab == 2) then
    MACROCOLLECTION.MACROS = MacroCollection_GENERICS;

  else
    MACROCOLLECTION.MACROS = MACROCOLLECTION.ALLMACROS;
  end

  -- Display each macro on selected list
  MacroCollection_ListMacros();

  -- Unselect macro
  MacroCollection_SelectMacro(false);
end

function MacroCollection_ResetContainer()
  MacroCollection_UncheckAllMacros();

  -- Get childrens
  local MacroBtns = { MacroCollection_ButtonContainer:GetChildren() };

  -- Hide them
  for x=1, table.getn(MacroBtns), 1 do
    MacroBtns[x]:Hide();
  end
end

function MacroCollection_ListMacros()
  local macroButtonName, macroButton, macroIcon;
  local firstPoint, relativeTo, secondPoint, marginLeft, marginTop;
  local counter = 1;

  for x=1, table.getn(MACROCOLLECTION.MACROS), 1
  do
    -- Define button Name
    local newID = MACROCOLLECTION.SELECTEDTAB .. MACROCOLLECTION.IDSPACER .. x;

    macroButtonName = "MacroCollection_MacroButton_" .. newID;

    -- Reuse or Create button
    if (_G[macroButtonName]) then
      macroButton = _G[macroButtonName];
    else
      macroButton = CreateFrame("CheckButton", macroButtonName, MacroCollection_ButtonContainer, "MacroButtonTemplate");
      macroButton:SetID(newID);
    end

    -- Set size
    macroButton:SetSize(48, 48);

    -- Get and set icon
    if (type(MACROCOLLECTION.MACROS[x]) == "table") then
      macroIcon = MacroCollection_GetMacroIcon(MACROCOLLECTION.MACROS[x]);

      if (macroIcon ~= "") then
        macroButton.icon = macroButton:CreateTexture(macroButtonName .. "_Icon", "ARTWORK");
        macroButton.icon:SetWidth(38);
        macroButton.icon:SetHeight(38);
        macroButton.icon:SetPoint("TOPLEFT", 5, -6);
        macroButton.icon:SetTexture(macroIcon);
      end
    end

    -- Calc buutton position
    if ( x == 1 ) then
      firstPoint = "TOPLEFT";
      relativeTo = MacroContainer_ButtonContainer;
      secondPoint = "TOPLEFT";
      marginLeft = 28;
      marginTop = -180;
		elseif ( mod(x, 6) == 1 ) then
      firstPoint = "TOP";
      relativeTo = "MacroCollection_MacroButton_" .. MACROCOLLECTION.SELECTEDTAB .. MACROCOLLECTION.IDSPACER .. (x - 6);
      secondPoint = "BOTTOM";
      marginLeft = 0;
      marginTop = 0;
		else
      firstPoint = "LEFT";
      relativeTo = "MacroCollection_MacroButton_" .. MACROCOLLECTION.SELECTEDTAB .. MACROCOLLECTION.IDSPACER .. (x - 1);
      secondPoint = "RIGHT";
      marginLeft = 0;
      marginTop = 0;
		end

    -- Set relative position
    macroButton:SetPoint(firstPoint, relativeTo, secondPoint, marginLeft, marginTop);

    -- Set onClick functionality
    macroButton:SetScript("OnClick", function(self, button, down)
      MacroCollection_SelectMacro(x);
    end);

    -- Show it
    macroButton:Show();

    -- Increase counter
    counter = counter + 1;
  end

  -- Set container's height
  local newHeight = upperRound(table.getn(MACROCOLLECTION.MACROS) / 6);
  MacroCollection_ButtonContainer:SetHeight(newHeight * 48);
end

function MacroCollection_SelectMacro(macroID)
  -- Save selection
  MACROCOLLECTION.SELECTEDMACRO = MACROCOLLECTION.MACROS[macroID];

  -- When a macro is selected
  if (MACROCOLLECTION.SELECTEDMACRO) then
    MacroCollection_UncheckAllMacros();

    local newID = MACROCOLLECTION.SELECTEDTAB .. MACROCOLLECTION.IDSPACER .. macroID;
    local macroButtonName = "MacroCollection_MacroButton_" .. newID;

    local macroButton = _G[macroButtonName];
    local selectedName = MACROCOLLECTION.SELECTEDMACRO.name;
    local selectedDesc = MACROCOLLECTION.SELECTEDMACRO.description;
    local selectedClass = MACROCOLLECTION.SELECTEDMACRO.class;
    local selectedIcon = MacroCollection_GetMacroIcon(MACROCOLLECTION.SELECTEDMACRO);

    -- Check selected macro
    macroButton:SetChecked(true);

    -- Set selected macro Name
    MacroCollection_SelectedMacroName:SetText(selectedName);
    MacroCollection_SelectedMacroName:Show();

    -- Set selected macro Description
    MacroCollection_SelectedMacroDesc:SetText(selectedDesc);
    MacroCollection_SelectedMacroDesc:Show();

    -- Set selected macro BG
    MacroCollection_SelectedMacroBg:SetTexture(selectedIcon);
    MacroCollection_SelectedMacroBg:Show();

    -- Enable buttons
    if (selectedClass == "none") then
      MacroCollection_AccountBtn:Enable();
      MacroCollection_CharBtn:Enable();

    elseif (selectedClass == MACROCOLLECTION.ClassIndex) then
      MacroCollection_CharBtn:Enable();
    end

  -- No macro is selected
  else
    MacroCollection_SelectedMacroBg:Hide();
    MacroCollection_SelectedMacroName:Hide();
    MacroCollection_SelectedMacroDesc:Hide();

    MacroCollection_AccountBtn:Disable();
    MacroCollection_CharBtn:Disable();
  end
end

function MacroCollection_UncheckAllMacros()
  MacroCollection_AccountBtn:Disable();
  MacroCollection_CharBtn:Disable();

  for x=1, table.getn(MACROCOLLECTION.MACROS), 1
  do
    local buttonName = "MacroCollection_MacroButton_" .. MACROCOLLECTION.SELECTEDTAB .. MACROCOLLECTION.IDSPACER .. x;
    if (_G[buttonName]) then
      _G[buttonName]:SetChecked(false);
    end
  end
end

function MacroCollection_GetMacroIcon(_macro)
  local macroIcon;

  if (_macro and _macro.itemID) then
    macroIcon = GetItemIcon(_macro.itemID);
  elseif (_macro and _macro.spellID) then
    macroIcon = GetSpellTexture(_macro.spellID);
  end

  return macroIcon;
end

function MacroCollection_GetAllMacros()
  local counter = 1;
  local allMacros = {};

  for k,v in pairs(MacroCollection_GENERICS) do
    allMacros[counter] = v;
    counter = counter + 1;
  end
  for k,v in pairs(MacroCollection_WARRIOR) do
    allMacros[counter] = v;
    counter = counter + 1;
  end
  for k,v in pairs(MacroCollection_PALADIN) do
    allMacros[counter] = v;
    counter = counter + 1;
  end
  for k,v in pairs(MacroCollection_HUNTER) do
    allMacros[counter] = v;
    counter = counter + 1;
  end
  for k,v in pairs(MacroCollection_ROGUE) do
    allMacros[counter] = v;
    counter = counter + 1;
  end
  for k,v in pairs(MacroCollection_PRIEST) do
    allMacros[counter] = v;
    counter = counter + 1;
  end
  for k,v in pairs(MacroCollection_DEATHKNIGHT) do
    allMacros[counter] = v;
    counter = counter + 1;
  end
  for k,v in pairs(MacroCollection_SHAMAN) do
    allMacros[counter] = v;
    counter = counter + 1;
  end
  for k,v in pairs(MacroCollection_MAGE) do
    allMacros[counter] = v;
    counter = counter + 1;
  end
  for k,v in pairs(MacroCollection_WARLOCK) do
    allMacros[counter] = v;
    counter = counter + 1;
  end
  for k,v in pairs(MacroCollection_MONK) do
    allMacros[counter] = v;
    counter = counter + 1;
  end
  for k,v in pairs(MacroCollection_DRUID) do
    allMacros[counter] = v;
    counter = counter + 1;
  end
  for k,v in pairs(MacroCollection_DEMONHUNTER) do
    allMacros[counter] = v;
    counter = counter + 1;
  end

  return allMacros;
end

function MacroCollection_AddToAccount()
  CreateMacro(
    MACROCOLLECTION.SELECTEDMACRO.name,
    MacroCollection_GetMacroIcon(MACROCOLLECTION.SELECTEDMACRO),
    MACROCOLLECTION.SELECTEDMACRO.macro,
    false
  );

  print("<" .. MACROCOLLECTION.SELECTEDMACRO.name .. "> " .. L["was added"] .. " " .. L["to"] .. " " .. L["your"] .. " " .. L["account"] .. ".");
end

function MacroCollection_AddToCharacter()
  CreateMacro(
    MACROCOLLECTION.SELECTEDMACRO.name,
    MacroCollection_GetMacroIcon(MACROCOLLECTION.SELECTEDMACRO),
    MACROCOLLECTION.SELECTEDMACRO.macro,
    true
  );

  print("<" .. MACROCOLLECTION.SELECTEDMACRO.name .. "> " .. L["was added"] .. " " .. L["to"] .. " " .. L["your"] .. " " .. L["character"] .. ".");
end



-- Slash commands
function MacroCollection_RegisterShashCommands()
  -- Supported Commands
  SLASH_MACROCOLLECTION1 = "/mc";
  SLASH_MACROCOLLECTION2 = "/macrocollection";

  -- Handler
  function SlashCmdList.MACROCOLLECTION(msg, editbox)

    if (msg == 'open') then
      MacroCollection_Show();
    end

    if (msg == 'close') then
      MacroCollection_Hide();
    end

    if (msg == '') then
      MacroCollection_Toggle();
    end
  end
end
function MacroCollection_RegisterEvents()

  MacroCollection:SetScript("OnEvent",
    function(self, event, ...)
    end
  )
end


-- EVENTS
-- Close button
function MacroCollection_Close_OnClick(self)
  MacroCollection_Hide();
end

-- Tabs
function MacroCollection_Tab_OnLoad(self)
  PanelTemplates_TabResize(self, 0);
  _G[self:GetName().."HighlightTexture"]:SetWidth(self:GetTextWidth() + 31);
end
function MacroCollection_Tab_OnClick(self, button, down)
  PlaySound("igMainMenuOptionCheckBoxOn");
  PanelTemplates_SetTab(MacroCollection, self:GetID());
  MacroCollection_UpdatePageContent(self:GetID());
  MacroCollection_Container:SetVerticalScroll(0);
end

-- Map icon
-- Place the icon properly
function MacroCollection_MapIcon_Reposition()
  MacroCollection_MapIcon:SetPoint("TOPLEFT", "Minimap", "TOPLEFT", 52 -(80 * cos(MACROCOLLECTION.SETTINGS.MinimapPos)), (80 * sin(MACROCOLLECTION.SETTINGS.MinimapPos)) - 52)
end
function MacroCollection_MapIcon_OnClick(self, button, down)
  if (button == "LeftButton") then
    MacroCollection_Toggle();
  elseif (MACROCOLLECTION.SETTINGS.debug == true) then
    ReloadUI();
  end
end
function MacroCollection_MapIcon_OnDragStart(self)
  self:LockHighlight();
  MacroCollection_MapIcon_DraggingFrame:Show();
end
function MacroCollection_MapIcon_OnDragStop(self)
  self:UnlockHighlight();
  MacroCollection_MapIcon_DraggingFrame:Hide();
end
-- Move minimap icon when Draggin
function MacroCollection_MapIcon_OnUpdate(self)
	local xpos,ypos = GetCursorPosition();
	local xmin, ymin = Minimap:GetLeft(), Minimap:GetBottom();

	xpos = xmin - xpos / UIParent:GetScale() + 70;
	ypos = ypos / UIParent:GetScale() - ymin - 70;

	MACROCOLLECTION.SETTINGS.MinimapPos = math.deg(math.atan2(ypos,xpos));
	MacroCollection_MapIcon_Reposition();
end
-- When The icon loads
function MacroCollection_MapIcon_OnLoad(self)
  self:RegisterForClicks("LeftButtonUp","RightButtonUp");
  self:RegisterForDrag("LeftButton","RightButton");
end

-- Macro Container
function MacroCollection_Container_OnScroll(self, offset)
  -- Save maxScroll
  local maxScroll = MacroCollection_Container:GetVerticalScrollRange();

  if (offset == 0) then
    -- Disable scrollup button

  elseif (offset == maxScroll) then
    -- Disable scrolldown button

  else
    -- Enable both

  end

  -- Move Scroll list
  local newID = MACROCOLLECTION.SELECTEDTAB .. MACROCOLLECTION.IDSPACER .. "1";
  local macroButtonName = "MacroCollection_MacroButton_" .. newID;

  local macroButton = _G[macroButtonName];
  macroButton:SetPoint("TOPLEFT", MacroCollection_Container, "TOPLEFT", 0, offset);
end

function MacroCollection_OnShow(self)
  -- Select tab 1
  MacroCollection_UpdatePageContent(1);

  -- Sound
  PlaySound("igCharacterInfoOpen");
end
function MacroCollection_OnHide(self)
	PlaySound("igCharacterInfoClose");
end
function MacroCollection_OnLoad(self)
  if ( MacroCollection ) then
    -- Set variables
    MACROCOLLECTION.CLASS = UnitClass("Player");
    MACROCOLLECTION.ALLMACROS = MacroCollection_GetAllMacros();

    -- Set textures
    MacroCollectionPortrait:SetTexture(MACROCOLLECTION.ICON);
    MacroCollection_MapIcon_Icon:SetTexture(MACROCOLLECTION.ICON);

    -- Register things/events
    MacroCollection_RegisterShashCommands();
    MacroCollection_RegisterEvents();

    -- Set UI Elements
    MacroCollectionTab1:SetText(MACROCOLLECTION.CLASS);
    PanelTemplates_TabResize(MacroCollectionTab1, 0);
    MacroCollectionTab2:SetText(MACROCOLLECTION.NOCLASS);
    PanelTemplates_TabResize(MacroCollectionTab2, 0);
    MacroCollectionTab3:SetText(MACROCOLLECTION.ALL);
    PanelTemplates_TabResize(MacroCollectionTab3, 0);
  	PanelTemplates_SetNumTabs(MacroCollection, 3);

    MacroCollection_WindowTitle:SetText(MACROCOLLECTION.TITLE);
    MacroCollection_AccountBtn:SetText(MACROCOLLECTION.ACCOUNT);
    MacroCollection_UpdatePageContent(1);
    MacroCollection_MapIcon_Reposition();

    if (MACROCOLLECTION.SETTINGS.debug) then
      MacroCollection_Show();
    end
  end
end
